@@ -14,6 +14,17 @@ import matplotlib.pyplot as plt
 import ibm_boto3
 from botocore.client import Config
 import statsmodels.api as sm
+from statsmodels.tsa.seasonal import STL
+
+icos_client = ibm_boto3.client(service_name='s3',
+    ibm_api_key_id='rYQmTc0U-gvfzOxV69jbUED3Fcy5QkuOL942TNeAWpT0',
+    ibm_auth_endpoint="https://iam.ng.bluemix.net/oidc/token",
+    config=Config(signature_version='oauth'),
+    endpoint_url='https://s3.us-east.cloud-object-storage.appdomain.cloud')
+
+bucketName='magna-formet-bucket'
+bucketName_model='magna-mig-models-data'
+
 
 #-----------------------------
 def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feature_data_normal):
@@ -31,42 +42,54 @@ def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feat
     DR_file_df = DR_file_df.drop_duplicates(subset =['PSN','WeldID'],keep='first',inplace=False)
 
     DR_PSN_red_list   = DR_file_df['PSN'].loc[(DR_file_df.WeldID == weld_id) & (DR_file_df.deviceId == deviceId) & (DR_file_df.Level == 'red')].tolist()
-    DR_PSN_green_list = DR_file_df['PSN'].loc[(DR_file_df.WeldID == weld_id) & (DR_file_df.deviceId == deviceId) & (DR_file_df.Level != 'red')].tolist()
+    DR_PSN_green_list = DR_file_df['PSN'].loc[(DR_file_df.WeldID == weld_id) & (DR_file_df.deviceId == deviceId) & (DR_file_df.Level == 'green')].tolist()
+    DR_PSN_yellow_list = DR_file_df['PSN'].loc[(DR_file_df.WeldID == weld_id) & (DR_file_df.deviceId == deviceId) & (DR_file_df.Level == 'yellow')].tolist()
     DR_PSN_red_list = list(set(DR_PSN_red_list))
     DR_PSN_red_list.sort()
     DR_PSN_green_list = list(set(DR_PSN_green_list))
     DR_PSN_green_list.sort()
+    DR_PSN_yellow_list = list(set(DR_PSN_yellow_list))
+    DR_PSN_yellow_list.sort()
     
-    #get validation data 
+    #get RED validation data 
     validate_df = feature_data_original.loc[lambda x: (x.LPSN.isin(DR_PSN_red_list)) & (x.weld_id == weld_id),:]   
     validate_df['label'] = 1
     Data_PSN_for_red_list = validate_df['LPSN'].tolist()
     Data_PSN_for_red_list = list(set(Data_PSN_for_red_list))
     Data_PSN_for_red_list.sort()
-
-    
     Data_WRI_for_red_list = validate_df['weld_record_index'].apply(lambda y: int(y)).tolist()
     Data_WRI_for_red_list = list(set(Data_WRI_for_red_list))
     Data_WRI_for_red_list.sort()
     
+    # Green      
     validate_green_df = feature_data_original.loc[lambda x: (x.LPSN.isin(DR_PSN_green_list)) & (x.weld_id == weld_id),:]   
     validate_green_df['label'] = 0
-    Data_PSN_for_green_list = list()
     Data_PSN_for_green_list = validate_green_df['LPSN'].tolist()
     Data_PSN_for_green_list = list(set(Data_PSN_for_green_list))
     Data_PSN_for_green_list.sort()
     Data_WRI_for_green_list = validate_green_df['weld_record_index'].apply(lambda y: int(y)).tolist()
     Data_WRI_for_green_list = list(set(Data_WRI_for_green_list))
     Data_WRI_for_green_list.sort()
-    
+
+    # Yellow
+    validate_yellow_df = feature_data_original.loc[lambda x: (x.LPSN.isin(DR_PSN_yellow_list)) & (x.weld_id == weld_id),:]   
+    validate_yellow_df['label'] = 0  # .loc[row_indexer,col_indexer] 
+    Data_PSN_for_yellow_list = validate_yellow_df['LPSN'].tolist()
+    Data_PSN_for_yellow_list = list(set(Data_PSN_for_yellow_list))
+    Data_PSN_for_yellow_list.sort()
+    Data_WRI_for_yellow_list = validate_yellow_df['weld_record_index'].apply(lambda y: int(y)).tolist()
+    Data_WRI_for_yellow_list = list(set(Data_WRI_for_yellow_list))
+    Data_WRI_for_yellow_list.sort()
+
+    # Cancat validation df
+    validate_green_df = pd.concat([validate_green_df, validate_yellow_df])    
     validate_df = pd.concat([validate_df, validate_green_df])
     validate_df = validate_df.drop_duplicates(subset ="weld_record_index",keep='first',inplace=False)
 
     Data_PSN_for_all_list = validate_df['LPSN'].tolist()
     Data_PSN_for_all_list = list(set(Data_PSN_for_all_list))
     Data_PSN_for_all_list.sort()
-    
-    
+        
     Data_WRI_for_all_list = validate_df['weld_record_index'].apply(lambda y: int(y)).tolist()
     Data_WRI_for_all_list = list(set(Data_WRI_for_all_list))
     Data_WRI_for_all_list.sort()
@@ -76,7 +99,10 @@ def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feat
         PSN_WRI_dict[Data_WRI_for_all_list[i]] = Data_PSN_for_all_list[i]
     #print(PSN_WRI_dict)
     print('Data_PSN_for_all_list:',len(Data_PSN_for_all_list))
-    print('Problem point (weld_record_index) list = ',len(Data_WRI_for_all_list))
+    print('All WRI point (weld_record_index) list = ',len(Data_WRI_for_all_list))
+    print('Red WRI point list = ',len(Data_WRI_for_red_list))
+    print('Green WRI point list = ',len(Data_WRI_for_green_list))
+    print('Yellow WRI point list = ',len(Data_WRI_for_yellow_list))
     validation_report_df= DR_file_df.loc[lambda x: (x.PSN.isin(Data_PSN_for_all_list)) & (x.WeldID == weld_id),:]
     display(validation_report_df[['Date','PSN','FailureType','Level','WeldID','ToolID']])
     #
@@ -101,11 +127,14 @@ def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feat
                     'DR_file_df':DR_file_df,
                     'DR_PSN_red_list':DR_PSN_red_list,
                     'DR_PSN_green_list':DR_PSN_green_list,
+                    'DR_PSN_yellow_list':DR_PSN_yellow_list,
                     'PSN_WRI_dict':PSN_WRI_dict,
                     'Data_WRI_for_red_list':Data_WRI_for_red_list,
-                    'Data_PSN_for_red_list':Data_PSN_for_red_list,
                     'Data_WRI_for_green_list':Data_WRI_for_green_list,
+                    'Data_WRI_for_yellow_list':Data_WRI_for_yellow_list,
+                    'Data_PSN_for_red_list':Data_PSN_for_red_list,
                     'Data_PSN_for_green_list':Data_PSN_for_green_list,
+                    'Data_PSN_for_yellow_list':Data_PSN_for_yellow_list,
                     'Data_WRI_for_all_list':Data_WRI_for_all_list,
                     'Data_PSN_for_all_list':Data_PSN_for_all_list,
                     'validate_df':validate_df,
@@ -346,6 +375,7 @@ def print_and_plot_anomaly_score(model_id,model_name,df_join_TF,Problem_object,a
     PSN_WRI_dict = Problem_object['PSN_WRI_dict']
     Data_WRI_for_red_list = Problem_object['Data_WRI_for_red_list']
     Data_WRI_for_green_list = Problem_object['Data_WRI_for_green_list']
+    Data_WRI_for_yellow_list = Problem_object['Data_WRI_for_yellow_list']
     Data_WRI_for_all_list = Problem_object['Data_WRI_for_all_list']
    
     print('percentile_point_95 : ',percentile_point_95)
@@ -360,41 +390,46 @@ def print_and_plot_anomaly_score(model_id,model_name,df_join_TF,Problem_object,a
     normal_green_list = list(set(normal_below_threshold_list) & set(Data_WRI_for_green_list))
     num_normal_below_threshold = len(normal_below_threshold_list)
     num_normal_green_list = len(normal_green_list)
-    
+    # yellow
+    yellow_above_threshold = list(set(anomaly_above_threshold_list) & set(Data_WRI_for_yellow_list))
+    num_yellow_above_threshold = len(yellow_above_threshold)    
     
     # plot anomaly score
+    plt.figure(figsize=(20,4))
+    Y = anomaly_score.loc[: , 0]
+    X = list(range(len(Y)))
+    plt.scatter(X, Y) 
+    #plt.plot(anomaly_score)
+    plt.title('Anomaly Score of each Weld based on Model ' + model_name)
+    plt.xlabel('Observation')
+    plt.ylabel('Anomaly Score')
+    plt.axhline(y=percentile_point_95, ls="--", c="red")
+    plt.axhline(y=anomaly_threshold, ls="--", c="yellow")
     if (plot_anomaly_score == 'YES'):
-        plt.figure(figsize=(20,4))
-        Y = anomaly_score.loc[: , 0]
-        X = list(range(len(Y)))
-        plt.scatter(X, Y) 
-        #plt.plot(anomaly_score)
-        plt.title('Anomaly Score of each Weld based on Model ' + model_name)
-        plt.xlabel('Observation')
-        plt.ylabel('Anomaly Score')
-        plt.axhline(y=percentile_point_95, ls="--", c="red")
-        plt.axhline(y=anomaly_threshold, ls="--", c="yellow")
         # bad welds:
         plot_problem_data(Data_WRI_for_red_list, df_join_TF, 'anomaly_score', "red")
         # normal welds"
         plot_problem_data(Data_WRI_for_green_list, df_join_TF , 'anomaly_score', "green")
+        plot_problem_data(Data_WRI_for_yellow_list, df_join_TF , 'anomaly_score', "orange")        
+    plt.show()
         
-        plt.show()
-        
+    period = 9 
+    stl_res = STL(anomaly_score[0],period=period,robust=True).fit()
+    plt.figure(figsize=(20,4))
+    plt.plot(stl_res.trend)
+    plt.title('Anomaly Score Trend based on Model ' + model_name)
+    plt.xlabel('Time Steps')
+    plt.ylabel('Anomaly Trend')
+
+    plt.axhline(y=percentile_point_95, ls="--", c="red")
+    plt.axhline(y=anomaly_threshold, ls="--", c="yellow")
     if (plot_anomaly_score == 'YES'):
-        plt.figure(figsize=(20,4))
-        #plt.plot(anomaly_score)
-        plt.title('Anomaly Score of each Weld based on Model ' + model_name)
-        plt.xlabel('Observation')
-        plt.ylabel('Anomaly Score')
-        plt.axhline(y=percentile_point_95, ls="--", c="red")
-        plt.axhline(y=anomaly_threshold, ls="--", c="yellow")
         # bad welds:
         plot_problem_data(Data_WRI_for_red_list, df_join_TF, 'anomaly_score', "red")
         # normal welds"
         plot_problem_data(Data_WRI_for_green_list, df_join_TF , 'anomaly_score', "green")
-        
-        plt.show()
+        plot_problem_data(Data_WRI_for_yellow_list, df_join_TF , 'anomaly_score', "orange")
+    plt.show()
     
     
     # print
@@ -415,13 +450,15 @@ def print_and_plot_anomaly_score(model_id,model_name,df_join_TF,Problem_object,a
     num_total_validation = len(Data_WRI_for_all_list)
     num_total_red = len(Data_WRI_for_red_list)
     num_total_green = len(Data_WRI_for_green_list)
+    num_total_yellow = len(Data_WRI_for_yellow_list)
     
     num_green_df = num_green_df.merge(df_join_PSN_TF[['PSN','anomaly_score']],on = ['PSN'], how = 'left')
     num_green_df = num_green_df.drop_duplicates(subset ="PSN",keep='first',inplace=False)
     
-    print('=========number of anomaly =============== \n total test #=',num_total_test,', total validation #=', num_total_validation,', total red=',num_total_red,', total green=',num_total_green)
+    print('=========number of anomaly =============== \n total test #=',num_total_test,', total validation #=', num_total_validation,', total red=',num_total_red,', total green=',num_total_green,', total yellow= ',num_total_yellow)
     print('# red above percentile_95:',num_red_points_above_95,', bad num above threshold:',num_anomaly_threshold, ', # welds above percentile_95:',num_anomaly_above_95)
-    print('---------number of normal ---------------- \n green below threshold:',num_normal_green_list,', total num welds below threshold:',num_normal_below_threshold)
+    print('---------number of normal ---------------- \n green below threshold:',num_normal_green_list,', total num welds below threshold:',num_normal_below_threshold,', # yellow above threshold:',num_yellow_above_threshold)
+    
     display(num_anomaly_df[['Date','PSN','FailureType','anomaly_score','Level','ToolID']])
     print('--------------------------------\n')
     display(num_green_df[['Date','PSN','FailureType','anomaly_score','Level','ToolID']])
